const Apify = require('apify');
const moment = require('moment');

const { crawlers } = Apify.client;

async function getCrawlerResults(executionId, count = 0) {
  try {
    const now = Date.now();
    const { status } = await crawlers.getExecutionDetails({ executionId });
    // Throttle requests
    const throttleTime = Date.now() - now + 300;
    await new Promise((resolve) => setTimeout(resolve, throttleTime));
    if (status !== 'SUCCEEDED' && count < 200) {
      console.log(`${status}...`);
      return getCrawlerResults(executionId, count + 1);
    }
    console.log(`${status}!`);
    const { items } = await crawlers.getExecutionResults({ executionId });
    return items;
  } catch (err) {
    throw err;
  }
}

async function callCrawler(crawlerId) {
  try {
    const { _id: executionId } = await crawlers.startExecution({
      crawlerId
    });
    return getCrawlerResults(executionId);
  } catch (err) {
    throw err;
  }
}

Apify.main(async () => {
  const input = await Apify.getValue('INPUT');
  if (!input && !input.crawlerId) {
    throw Error('A crawlerId as input is required.');
  }

  const [today] = moment().toISOString().split`T`;
  console.log(today);

  const kvStore = await Apify.openKeyValueStore(today);
  const crawlerResults = await callCrawler(input.crawlerId);

  // Parse & concat pageFunctionResult
  const results = crawlerResults.reduce(
    (acc, { pageFunctionResult }) => acc.concat(pageFunctionResult),
    []
  );

  await kvStore.setValue('RESULTS', results);
  return console.log('Done.');
});
